//========================================================
//HI-MD Renderer Application - Settings store functions
//
//Written by Mark Collier.
//http://www.marcnetsystem.co.uk/
//
//This program is freeware and may be distributed in part or 
//in whole as long as a the author is credited clearly some way in 
//both the source code and in some visible way when the program 
//is running (eg:in a help screen or version info screen ).

//This notice must also appear in any source file that contains
//any part of the code in this file. 

//I (the author) take no responsability for the loss, transfer, corruption
//duplication, modification, or addition of legal or illegal data in any way. 
//
//Thanks for reading

//MarC
//=================================================================
#include <windows.h>
#include "internal.h"
#include "stdio.h"
//===========================Globals================================

static DWORD bKeepTmpFile = FALSE;
static DWORD bDebug = FALSE;
static DWORD dwBlockSize = 30;
static DWORD dwSkipSize = 10000000;
static DWORD dwAllowedDiff = 5;
static DWORD bUseAvging = TRUE;
static DWORD bReturnAvgIfLongRpt = TRUE;
static DWORD dwSmoothAnlWindow = 32;
static DWORD dwSmoothAvgWindow = 8;
static DWORD bEnableSmoothing = TRUE;

//========================Main setting handler===============================
static void SettingHandler(DWORD dwIndex,DWORD * lpdwValue,BOOL bSetValue)
{
	LPDWORD ptr = NULL;
	
	switch(dwIndex)
	{
		case SETTING_DEBUG:
			ptr = &bDebug;
			break;
		case SETTING_KEEPTMPFILE:
			ptr = &bKeepTmpFile;
			break;
		case SETTING_BLOCKSIZE:
			ptr = &dwBlockSize;
			break;
		case SETTING_SKIPSIZE:
			ptr = &dwSkipSize;
			break;
		case SETTING_ALLOWEDDIFF:
			ptr = &dwAllowedDiff;
			break;
		case SETTING_USEAVGING:
			ptr = &bUseAvging;
			break;
		case SETTING_NOIGNORELONGRPTS:
			ptr = &bReturnAvgIfLongRpt;
			break;
		case SETTING_SMOOTH_AVGING_WINDOW:
			ptr = &dwSmoothAvgWindow;
			break;
		case SETTING_SMOOTH_ANALYSIS_WINDOW:
			ptr = &dwSmoothAnlWindow;
			break;
		case SETTING_ENABLESMOOOTHING:
			ptr = &bEnableSmoothing;
		default:
			break;
	}

	if(bSetValue)
	{
		*ptr = *lpdwValue;
	}
	else
	{
		*lpdwValue = *ptr;
	}
}

//========================Exported Functions=================================
DWORD CheckSettings(LPSTR errbuff)
{
	DWORD dwVal,i,tmp;

	for(i=1 ; i <= NUMSETTINGS; i++)
	{
		SettingHandler(i,&dwVal,FALSE);

		switch(i)
		{
			case SETTING_BLOCKSIZE:
				if((dwVal < 2) || (dwVal > 50))
				{
					sprintf(errbuff,"Block size must be more than 2 seconds and less than 50");
					return SETTING_BLOCKSIZE;
				}
				break;
			case SETTING_SKIPSIZE:
				SettingHandler(SETTING_BLOCKSIZE,&tmp,FALSE);
				if((dwVal < 1000000) || (dwVal >= (tmp * 10000000)))
				{
					sprintf(errbuff,"Overlap size must be more than 100 milliseconds and less than the block size");
					return SETTING_SKIPSIZE;
				}
				break;
			case SETTING_ALLOWEDDIFF:
				if((dwVal < 1) || (dwVal >= 128))
				{
					sprintf(errbuff,"Maximum difference must be between 1 and 127");
					return SETTING_ALLOWEDDIFF;
				}
				break;
			case SETTING_SMOOTH_ANALYSIS_WINDOW:
				if(bEnableSmoothing)
				{
					if((dwVal > 256) || ((dwVal % 2) != 0))
					{
						sprintf(errbuff,"Smoohing analysis window must be less than 256 and divisable by 2");
						return SETTING_SMOOTH_ANALYSIS_WINDOW;
					}
				}
				break;
			case SETTING_SMOOTH_AVGING_WINDOW:
				if(bEnableSmoothing)
				{
					if((dwVal < 2) || (dwVal > 256) || ((dwVal % 2) != 0) || (dwVal > dwSmoothAnlWindow) )
					{
						sprintf(errbuff,"Smoohing averaging window must be between 0 and 256, divisable by 2 and less than analysis window size");
						return SETTING_SMOOTH_AVGING_WINDOW;
					}
				}
				
			default: //no checking
				break;
		}

	}

	return 0;//all ok
}


void SetSetting(DWORD dwIndex,DWORD dwValue)
{
	DWORD value = dwValue;
	SettingHandler(dwIndex,&value,TRUE);
}


DWORD GetSetting(DWORD dwIndex)
{
	DWORD dwValue = 0;

	SettingHandler(dwIndex,&dwValue,FALSE);
	return dwValue;
}



BOOL CheckFileIsOK(LPSTR name,BOOL bNewFile)
{
	HANDLE hFile = CreateFile(name, 
							  bNewFile ? GENERIC_READ | GENERIC_WRITE : GENERIC_READ ,
							  bNewFile ? 0 : FILE_SHARE_READ | FILE_SHARE_WRITE,
							  NULL,
							  bNewFile ? CREATE_ALWAYS : OPEN_EXISTING,
							  FILE_ATTRIBUTE_NORMAL,NULL);

	if(hFile != INVALID_HANDLE_VALUE)
	{
		CloseHandle(hFile);
		return TRUE;
	}

	return FALSE;
}

